@@ -16,6 +16,18 @@
 #include <arpa/inet.h>
 #endif
 
+#if defined(WIN32)
+/*
+ * Since setup.py uses a '-WX' in the CFLAGS (treat warnings as errors),
+ * the below will turn off some warnings when using MS-SDK 8.1+.
+ * This MUST be defined before including <winsock2.h> via the libcurl
+ * headers.
+ */
+# if !defined(_WINSOCK_DEPRECATED_NO_WARNINGS)
+#  define _WINSOCK_DEPRECATED_NO_WARNINGS
+# endif
+#endif
+
 #include <curl/curl.h>
 #include <curl/easy.h>
 #include <curl/multi.h>
@@ -34,8 +46,8 @@
 #  define EAFNOSUPPORT 97
 # endif
 
-PYCURL_INTERNAL SOCKET
-dup_winsock(SOCKET sock, const struct curl_sockaddr *address);
+PYCURL_INTERNAL int
+dup_winsock(int sock, const struct curl_sockaddr *address);
 #endif
 
 /* The inet_ntop() was added in ws2_32.dll on Windows Vista [1]. Hence the
@@ -107,6 +119,10 @@ pycurl_inet_ntop (int family, void *addr, char *string, size_t string_size);
 #define HAVE_CURL_7_21_5
 #endif
 
+#if LIBCURL_VERSION_NUM >= 0x071600 /* check for 7.22.0 or greater */
+#define HAVE_CURL_7_22_0_OPTS
+#endif
+
 #if LIBCURL_VERSION_NUM >= 0x071800 /* check for 7.24.0 or greater */
 #define HAVE_CURLOPT_DNS_SERVERS
 #define HAVE_CURL_7_24_0
@@ -211,6 +227,17 @@ PYCURL_INTERNAL void pycurl_ssl_cleanup(void);
   #define PyInt_AsLong                 PyLong_AsLong
 #endif
 
+#define PYLISTORTUPLE_LIST 1
+#define PYLISTORTUPLE_TUPLE 2
+#define PYLISTORTUPLE_OTHER 0
+
+PYCURL_INTERNAL int
+PyListOrTuple_Check(PyObject *v);
+PYCURL_INTERNAL Py_ssize_t
+PyListOrTuple_Size(PyObject *v, int which);
+PYCURL_INTERNAL PyObject *
+PyListOrTuple_GetItem(PyObject *v, Py_ssize_t i, int which);
+
 /*************************************************************************
 // python 2/3 compatibility
 **************************************************************************/
